/* obscene little program to check that an ISO 9660 image could boot */
/* using El Torito extensions                                        */

/* If this check FAILS, it means the ISO image most likely WOULD NOT boot */
/* If this check PASSES, it means nothing. You still need to test boot */
/* I use it to see if it FAILS, if it does then I dont reboot my workstation */
/* to test boot it yet...                                               */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>  

/* bytes per iso 9660 sector */
#define SECTOR 2048

unsigned char sec[SECTOR];


void willnotboot( void )
{
  fprintf(stderr, "\nDue to errors, this image will not boot\n");
  exit(1);
}

main (argc, argv)
int argc;
char **argv;
{

  int i, chksum, f;
  int testnum;
  int bootcatoff, bootimageoff;

  /* open iso image (can be block device like /dev/cdrom too) */
  f = open(argv[1], O_RDONLY );
  if (f < 0 )
    {
      fprintf(stderr, "Cannot open iso image %s\n",argv[1]);
      fprintf(stderr, "Usage: checkboot isofile\n");
      perror("");
      exit(1);
    }

  /* seek to sector 17 (dec) and look for BVD */
  if (lseek(f, 17*SECTOR, SEEK_SET) != 17*SECTOR)
    {
      fprintf(stderr, "Iso image too small to be valid\n");
      willnotboot();
    }

  testnum = 1;

  /* read in sector */
  fprintf(stderr, "TEST %d: ",testnum);
  if (read(f, sec, SECTOR) != SECTOR)
    {
      fprintf(stderr, "FAIL - Cannot read boot volume descriptor\n");
      willnotboot();
    }

  fprintf(stderr,"SUCCESS - Read possible boot record volume descriptor ok...\n");
  testnum++;

  /* check for El Torito signature */
  fprintf(stderr, "TEST %d: ",testnum);
  if (strncmp((char *)(sec+7),"EL TORITO SPECIFICATION",24))
    {
      fprintf(stderr,"FAIL - No El Torito signature\n");
      willnotboot();
    }
  
  fprintf(stderr,"SUCCESS - El Torito signature found...\n");
  testnum++;

  /* look for boot catalog */
  fprintf(stderr, "TEST %d: ",testnum);
  bootcatoff = (unsigned int)sec[0x4a] << 24;
 bootcatoff += (unsigned int)sec[0x49] << 16;
  bootcatoff += (unsigned int)sec[0x48] << 8;
  bootcatoff += (unsigned int)sec[0x47];

  if (lseek(f, bootcatoff*SECTOR, SEEK_SET) != bootcatoff*SECTOR)
    {
      fprintf(stderr, "FAIL - cannot seek to boot catalog\n");
      willnotboot();
    }

  /* read in sector */
  if (read(f, sec, SECTOR) != SECTOR)
    {
      fprintf(stderr, "FAIL - Cannot read boot catalog, WILL NOT BOOT\n");
      willnotboot();
    }

  fprintf(stderr,"SUCCESS - Read possible boot catalog ok...\n");
  testnum++;

  /* see if first record is Validation Entry */
  fprintf(stderr, "TEST %d: ",testnum);
  
  /* check header ID */
  if (sec[0] != 1)
    {
      fprintf(stderr,"FAIL - Header ID wrong\n");
      willnotboot();
    }

  /* check platform */
  if (sec[1] != 0)
    {
      fprintf(stderr,"FAIL - Platform ID is not x86\n");
      willnotboot();
    }

  /* check checksum */
  chksum = 0;
  for (i=0; i<16; i++)
    if (i*2 != 28)
      chksum += sec[i*2] + (sec[i*2+1] * 256);

  /* negate it */
  chksum = -chksum;

  /* compare lower 2 bytes */
  if ((chksum & 0xffff) != (sec[28] + (sec[29] * 256)))
    {
      fprintf(stderr,"FAIL - Checksum is not correct\n");
      willnotboot();
    }

  fprintf(stderr,"SUCCESS - Validation entry ok\n");
  testnum++;

  /* now check Initial/default entry */
  fprintf(stderr, "TEST %d: ",testnum);

  if (sec[32] != 0x88)
    {
      fprintf(stderr,"FAIL - not set as bootable\n");
      willnotboot();
    }

  if ((sec[38] + (sec[39] << 8)) != 1)
    {
      fprintf(stderr,"FAIL - number of sectors in boot image != 1\n");
      willnotboot();
    }

  fprintf(stderr, "SUCCESS - boot catalog looks ok\n");
  testnum++;

  /* 
   * read in bootimage, see if SYSLINUX tag there
   * Also, see if LILO there.  Either one is a success.
   * DLT 14 MAY 1998
   */
  fprintf(stderr, "TEST %d: ",testnum);
  bootimageoff = (sec[43] << 24) + (sec[42] << 16) + (sec[41] << 8) + sec[40];

  if (lseek(f, bootimageoff*SECTOR, SEEK_SET) != bootimageoff*SECTOR)
    {
      fprintf(stderr, "FAIL - cannot seek to boot image\n");
      willnotboot();
    }

  if (read(f, sec, SECTOR) != SECTOR)
    {
      fprintf(stderr, "FAIL - Cannot read boot volume descriptor\n");
      willnotboot();
    }

  /* look for lilo or syslinux*/
  if (strncmp((char *)(sec+3), "SYSLINUX",8))
    {
      if (strncmp((char *)(sec+2), "LILO", 4))
	{
          /*fprintf(stderr, "%8s\n", (char *)(sec+3));*/
          fprintf(stderr, "FAIL - boot image doesn't appear to use SYSLINUX or LILO\n");
          willnotboot();
        }
    }

  fprintf(stderr,"SUCCESS - Syslinux or Lilo image found\n");
  testnum++;



  /* everything seems ok */

  fprintf(stderr,"\nALL TESTS PASSED\n\n");
  fprintf(stderr,"Looks like ISO image may boot\n");
  fprintf(stderr,"But PLEASE test it to be sure....\n");
}



  
